/* ESP32_gombvezérelt LED-ek
 *  
 * Két LED-et (GPIO23 és GPIO22) különböző ütemben villogtatunk. 
 * A villogás sebességát a GPIO21 bemenetre kötött nyomógombbal
 * vezéreljük: minden lenyomás megfelezi a villogás periódusidejét
 * (az egyszerűség kedvéért egyszerre, mindkét LED-ét).
 * Induláskor a piros LED 1000 ms-onként vált állapotot, a zöld LED
 * pedig 5000 ms-onként.
 * Ha a piros LED átbillenési ideje már kisebb, mint 200 ms, akkor 
 * a következő gombnyomáskor állítsuk vissza a kiindulási átbillenési időket!
 * Gondoljunk a nyomógomb pergésére! Egyszerű megoldásként csak 
 * 20 ms-onként vizsgáljuk meg a nyomógomb állapotát.
 * 
 * Arduino 1.8.15 és ESP32 2.0.0 kiadással tesztelve
 */
#define RED_LED   23
#define GREEN_LED 22
#define BUTTON    21
uint32_t nowTime, redTime, greenTime, buttonTime;
uint32_t redInterval = 1000, greenInterval = 5000;
boolean rState = HIGH, gState = HIGH, bState = HIGH;

void changeSpeed() {
  if (redInterval < 200) {
    greenInterval = 5000;
    redInterval = 1000;
  }
  else {
    greenInterval /= 2;
    redInterval /= 2;
  }
}

void setup() {
  pinMode(RED_LED, OUTPUT);
  pinMode(GREEN_LED, OUTPUT);
  pinMode(BUTTON, INPUT_PULLUP);
  nowTime = millis();
  redTime = nowTime + redInterval;
  greenTime = nowTime + greenInterval;
  buttonTime = nowTime + 20;
}

void loop() {
  nowTime = millis();
  if (nowTime > redTime) {
    rState = !rState;
    digitalWrite(RED_LED, rState);
    redTime = nowTime + redInterval;
  }
  if (nowTime > greenTime) {
    gState = !gState;
    digitalWrite(GREEN_LED, gState);
    greenTime = nowTime + greenInterval;
  }
  if (nowTime > buttonTime) {
    buttonTime = nowTime + 20;
    if (digitalRead(BUTTON) != bState) {
      bState = !bState;
      if (bState == LOW) changeSpeed();
    }
  }
}
